;; These examples exhibit the music model implemented in proto-event.
;; They are translations from Chapter 5 of Streams-Patterns-Events from
;; SuperCollider.

;; Composing with frequency
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.25
                       'freq (stream:sequence -1 300 400 500 700 900))))

;; Detuning the frequency
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.25
                       'detune -20
                       'freq (stream:sequence -1 300 400 500 700 900))))

;; Composing with MIDI notes
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.2
                       'midi-note (stream:shuffled-sequence -1 60 61 62 63 64 65 66 67))))

;; Composing by degree
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.15
                       'degree (stream:sequence -1
                                                (stream:shuffled-sequence 4 -7 3 0 2 4 7)
                                                (stream:sequence 1 0 1 2 3 4 5 6 7)))))

;; Same thing, different octave
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.15
                       'octave 4
                       'degree (stream:sequence -1
                                                (stream:shuffled-sequence 4 -7 3 0 2 4 7)
                                                (stream:sequence 1 0 1 2 3 4 5 6 7)))))

;; Different scale
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'scale '(0 2 3 5 7 8 10)
                       'dur 0.15
                       'degree (stream:sequence -1
                                                (stream:shuffled-sequence 4 -7 3 0 2 4 7)
                                                (stream:sequence 1 0 1 2 3 4 5 6 7)))))

;; Modal transpositions
(let ((notes (stream:sequence 1 (stream:shuffled-sequence 4 -7 3 0 2 4 7)
                              (stream:sequence 1 0 1 2 3 4 5 6 7))))
  (play (estream->unit-generator
         (estream:binder (stream:sequence -1
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'mtranspose 0)
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'mtranspose 1)
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'mtranspose 2))
                         'dur 0.15))))

;; Chromatic transpositions
(let ((notes (stream:sequence 1 (stream:shuffled-sequence 4 -7 3 0 2 4 7)
                              (stream:sequence 1 0 1 2 3 4 5 6 7))))
  (play (estream->unit-generator
         (estream:binder (stream:sequence -1
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'ctranspose 0)
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'ctranspose 3)
                                          (estream:binder (estream:proto-event)
                                                          'degree notes
                                                          'ctranspose -3))
                         'dur 0.15))))

;; Chords aren't supported yet.

;; 72 tone equal temperament.
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'steps-per-octave 72
                       'dur (stream:sequence -1
                                             (stream:sequence 5 0.45)
                                             (stream:sequence 12 0.15)
                                             (stream:sequence 5 0.45))
                       'note (stream:sequence -1
                                              ;; 1/1 7/6 3/2 7/4 9/8
                                              (stream:sequence 1
                                                               0 16 42 58 84
                                                               (stream:sequence
                                                                2 0 16 42 58 72 84)
                                                               0 16 42 58 84)
                                              ;; 1/1 6/5 3/2 9/5 9/8
                                              (stream:sequence 1
                                                               0 19 42 61 84
                                                               (stream:sequence
                                                                2 0 19 42 61 72 84)
                                                               0 19 42 61 84)
                                              ;; 1/1 5/4 3/2 15/8 9/8
                                              (stream:sequence 1
                                                               0 23 42 65 84
                                                               (stream:sequence
                                                                2 0 23 42 65 72 84)
                                                               0 23 42 65 84)
                                              ;; 1/1 9/7 3/2 27/14 9/8
                                              (stream:sequence 1
                                                               0 26 42 68 84
                                                               (stream:sequence
                                                                2 0 26 42 68 72 84)
                                                               0 26 42 68 84)))))

;; Some duration examples -- both using 'dur and 'legato.
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur (stream:geom 0.05 1.1 24)
                       'midi-note (stream:sequence -1 60 58))))

;; Doesn't seem to have any effect at the moment..
(play (estream->unit-generator
       (estream:binder (estream:proto-event)
                       'dur 0.2
                       'legato (stream:sequence 1
                                                (stream:series 0.05 0.05 40)
                                                (stream:series 2.05 -0.05 40))
                       'midi-note (stream:sequence -1 48 51 55 58 60 58 55 51))))
