;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;Soundscrape already has @code{mix}, which generates the unit
;;generators necessary to mix together a set of inputs. @code{mixer},
;;defined in this module, is a generator that makes it easy to
;;dynamically add new inputs for mixing. There is no mixer object per
;;se; only a collection of generators made by the @code{+} operator.
;;
;;; Code:

(define-module (soundscrape music mixer)
  :use-module (oop goops)
  :use-module (soundscrape element-support) ;; for support :)
  :use-module (soundscrape elements) ;; for the generics
  :use-module (soundscrape ugens) ;; for ss-spawn-adder
  :use-module (soundscrape gst-util)
  :use-module (soundscrape util)
  :use-module (scheme documentation)
  :use-module (gnome gstreamer) ;; <gst-bin>
  :export (mixer))

(define mixer-list (make-object-property))

(define (mixer rate . args)
  (let* ((outputs (mix args))
         (loutputs ((lambda (x) (if (list? x) x (list x))) outputs))
         (mixers (map get-parent loutputs)))
    (for-each (lambda (m) (set m 'ss-rate rate)) mixers)
    (for-each (lambda (p) (set! (mixer-list p) mixers)) loutputs)
    outputs))
(define-unit-generator! 'mixer mixer "Mixer" "Mixer")
    
(define (mixer:add-input mixer-pads pad-or-pads)
  (let ((channel-mixers (or (mixer-list (car mixer-pads)) (error "not a mixer" mixer-pads)))
        (pads ((lambda (x) (if (list? x) x (list x))) pad-or-pads)))
    (let lp ((pads pads)
             (adders (apply circular-list channel-mixers)))
      (if (not (null? pads))
          (begin
            (link (car pads) (get-pad (car adders) "sink%d"))
            (lp (cdr pads) (cdr adders)))))))
